@@ -3,7 +3,8 @@
 export MESON_BUILD_ROOT="$1"
 export MESON_SOURCE_ROOT="$2"
 export CARGO_TARGET_DIR="$MESON_BUILD_ROOT"/target
-export CARGO_HOME="$CARGO_TARGET_DIR"/cargo-home
+export CARGO_HOME="$MESON_BUILD_ROOT"/cargo-home
+
 FEATURES="$6"
 
 if [ $4 = "Devel" ]
