@@ -5,7 +5,6 @@ use gio::prelude::*;
 use glib::clone;
 use gtk::prelude::*;
 use log::info;
-use std::env;
 use std::{cell::RefCell, rc::Rc};
 
 pub struct Application {
@@ -15,7 +14,7 @@ pub struct Application {
 
 impl Application {
     pub fn new() -> Rc<Self> {
-        let app = gtk::Application::new(Some(config::APP_ID), gio::ApplicationFlags::FLAGS_NONE).unwrap();
+        let app = gtk::Application::new(Some(config::APP_ID), gio::ApplicationFlags::FLAGS_NONE);
 
         let application = Rc::new(Self {
             app,
@@ -61,7 +60,7 @@ impl Application {
             "next-page",
             clone!(@strong application => move |_, _| {
                 if let Some(window) = &*application.window.borrow().clone() {
-                    if window.paginator.borrow_mut().next().is_err() {
+                    if window.paginator.borrow_mut().try_next().is_none() {
                         window.widget.close();
                     }
                 }
@@ -73,7 +72,7 @@ impl Application {
             "previous-page",
             clone!(@strong application => move |_, _| {
                 if let Some(window) = &*application.window.borrow().clone() {
-                    if window.paginator.borrow_mut().previous().is_err() {
+                    if window.paginator.borrow_mut().try_previous().is_none() {
                         window.reset_tour();
                     }
                 }
@@ -89,19 +88,20 @@ impl Application {
             app.setup_css();
             app.setup_gactions(app.clone());
         }));
-        self.app.connect_activate(clone!(@weak app => move |gtk_app| {
-           let window = Window::new(&gtk_app);
-            gtk_app.add_window(&window.widget);
-            window.widget.present();
-            window.widget.show();
-            app.window.replace(Rc::new(Some(window)));
-        }));
+        self.app
+            .connect_activate(clone!(@weak app => move |gtk_app| {
+               let window = Window::new(&gtk_app);
+                gtk_app.add_window(&window.widget);
+                window.widget.present();
+                window.widget.show();
+                app.window.replace(Rc::new(Some(window)));
+            }));
     }
 
     fn setup_css(&self) {
         let p = gtk::CssProvider::new();
         gtk::CssProvider::load_from_resource(&p, "/org/gnome/Tour/style.css");
-        if let Some(screen) = gdk::Screen::get_default() {
+        if let Some(screen) = gdk::Screen::default() {
             gtk::StyleContext::add_provider_for_screen(&screen, &p, 500);
         }
     }
@@ -111,7 +111,6 @@ impl Application {
         info!("Version: {} ({})", config::VERSION, config::PROFILE);
         info!("Datadir: {}", config::PKGDATADIR);
 
-        let args: Vec<String> = env::args().collect();
-        self.app.run(&args);
+        self.app.run();
     }
 }
